@@ -2,175 +2,121 @@ import json
 import neo4j
 import base64
 
-class TrellisMessage(object):
 
-    def __init__(self, *, message_kind=None, sender=None, seed_id=None, previous_event_id=None):
-        self.message_kind = message_kind
+class MessageWriter(object):
+
+    def __init__(
+                 self,
+                 sender,
+                 seed_id,
+                 previous_event_id):
+
+        self.message_kind = "unspecifiedWriter"
         self.sender = sender
         self.seed_id = seed_id
         self.previous_event_id = previous_event_id
 
-        self.context = None
-        self.header = None
-        self.body = None
+    def format_json_header(self):
 
-    def parse_pubsub_message(self, event, context):
-        pubsub_message = base64.b64decode(event['data']).decode('utf-8')
-        data = json.loads(pubsub_message)
-
-        # Not sure what cases caused me to add this to db-query,
-        # but keeping it in because I don't see any harm
-        if type(data) == str:
-            logging.warn("Message data not correctly loaded as JSON. " +
-                         "Used eval to convert from string.")
-            data = eval(data)
-        
-        self.context = context
-        self.header = data['header']
-        self.body = data['body']
-        self.sender = self.header['sender']
-        
-        # The event_id is created by Pub/Sub or the message broker
-        # so only get it when parsing messages
-        self.event_id = int(context.event_id)
-        self.seed_id = int(self.header.get('seedId'))
-
-        
-        # If no seed specified, assume this is the seed event
-        if not self.seed_id:
-            self.seed_id = self.event_id
-
-    def format_json_message(self):
-        raise NotImplementedError
-
-
-class QueryRequest(TrellisMessage):
-
-    def __init__(self,
-                 * , # Begin keyword-only arguments
-                 sender=None, 
-                 seed_id=None, 
-                 previous_event_id=None,
-                 query_name=None,
-                 query_parameters={}, 
-                 write_transaction = False, 
-                 results_mode = "data", 
-                 split_results = True, 
-                 publish_results_to = []):
-        
-        super().__init__(
-                         message_kind="queryRequest",
-                         sender=sender, 
-                         seed_id=seed_id, 
-                         previous_event_id=previous_event_id)
-        
-        self.query_name = query_name
-        self.query_parameters = query_parameters
-        self.write_transaction = write_transaction
-        self.results_mode = results_mode
-        self.split_results = split_results
-        self.publish_results_to = publish_results_to
-
-    def parse_pubsub_message(self, event, context):
-        super().parse_pubsub_message(event, context)
-
-        if self.header['messageKind'] != "queryRequest":
-            return ValueError
-
-        # Required fields
-        self.previous_event_id = int(self.header['previousEventId'])
-        self.query_name = self.body['queryName']
-        self.write_transaction = self.body['writeTransaction']
-        self.results_mode = self.body['resultsMode']
-        self.split_results = self.body['splitResults']
-
-        # Optional fields
-        self.query_parameters = self.body.get('queryParameters')
-        self.publish_results_to = self.body.get('publishResultsTo')
-
-        if not isinstance(self.publish_results_to, list):
-            return ValueError
-
-    def format_json_message(self):
         message = {
            "header": {
                       "messageKind": self.message_kind,
                       "sender": self.sender,
                       "seedId": self.seed_id,
                       "previousEventId": self.previous_event_id,
-           },
-           "body": {
-                    # Trigger name should be named the same as the database query it trigger
-                    "queryName": self.query_name,
-                    "queryParameters": self.query_parameters,
-                    "writeTransaction": self.write_transaction,
-                    "resultsMode": self.results_mode,
-                    "splitResults": self.split_results,
-                    "publishResultsTo": self.publish_results_to
            }
         }
         return message
 
 
-class QueryResponse(TrellisMessage):
+class QueryRequestWriter(MessageWriter):
 
-    def __init__(self,
+    def __init__(
+                 self,
+                 *, # Begin keyword-only arguments
                  sender,
                  seed_id,
                  previous_event_id,
-                 *, # Begin keyword-only arguments
                  query_name,
-                 graph,
-                 result_summary):
-
-        if not isinstance(graph, neo4j.graph.Graph):
-            return TypeError
-        if not isinstance(result_summary, neo4j.ResultSummary):
-            return TypeError
+                 query_parameters,
+                 custom=False,
+                 query=None,
+                 write_transaction=False,
+                 publish_to=None,
+                 returns={}):
 
         super().__init__(
-                         message_kind = "queryResponse",
-                         sender = sender,
-                         seed_id = seed_id,
-                         previous_event_id = previous_event_id)
+                         sender,
+                         seed_id,
+                         previous_event_id)
 
+        self.message_kind = "queryRequest"
         self.query_name = query_name
-        self.graph = graph
-        self.result_summary = result_summary
-
-        self.nodes = []
-        for node in self.graph.nodes:
-            self.nodes.append(node)
-
-        self.relationships = []
-        for relationship in self.graph.relationships:
-            self.relationships.append(relationship)
-
-    def parse_pubsub_message(self, event, context):
-        super().parse_pubsub_message(event, context)
-
-        if self.header['messageKind'] != "queryResponse":
-            return ValueError
-
-        # Required fields
-        self.previous_event_id = int(self.header['previousEventId'])
-        self.query_name = self.body['queryName']
-
-        # Optional fields
-        self.query_parameters = self.body.get('queryParameters')
-        self.nodes = self.body.get('nodes')
-        self.relationships = self.body.get('relationships')
-        self.result_summary = self.body.get('resultSummary')
+        self.query_parameters = query_parameters
+        self.custom = custom
+        self.query = query
+        self.write_transaction = write_transaction
+        self.publish_to = publish_to
+        self.returns = returns
 
     def format_json_message(self):
+        message = super().format_json_header()
+        body = {
+                "body": {
+                    "queryName": self.query_name,
+                    "queryParameters": self.query_parameters,
+                    # Adding support for custom queries
+                    "custom": self.custom
+                }
+        }
+        message.update(body)
 
-        message = {
-           "header": {
-                      "messageKind": self.message_kind,
-                      "sender": self.sender,
-                      "seedId": self.seed_id,
-                      "previousEventId": self.previous_event_id,
-           },
+        if self.custom:
+            custom_fields = {
+                "query": self.query,
+                "writeTransaction": self.write_transaction,
+                "publishTo": self.publish_to,
+                "returns": self.returns
+            }
+            message['body'].update(custom_fields)
+        return message
+
+
+class QueryResponseWriter(MessageWriter):
+
+    def __init__(
+                 self,
+                 *,
+                 sender,
+                 seed_id,
+                 previous_event_id,
+                 query_name,
+                 result_summary,
+                 graph):
+
+        super().__init__(
+                         sender,
+                         seed_id,
+                         previous_event_id)
+
+        self.message_kind = "queryResponse"
+        self.query_name = query_name
+        self.result_summary = result_summary
+        self.graph = graph
+        self.nodes = []
+        self.relationships = []
+
+        if self.graph:
+            for node in self.graph.nodes:
+                self.nodes.append(node)
+
+            for relationship in self.graph.relationships:
+                self.relationships.append(relationship)
+
+    def format_json_message(self):
+        message = super().format_json_header()
+
+        body = {
            "body": {
                     "queryName": self.query_name,
                     "nodes": [self._get_node_dict(node) for node in self.nodes],
@@ -178,6 +124,7 @@ class QueryResponse(TrellisMessage):
                     "resultSummary": self._get_result_summary_dict(self.result_summary)
            }
         }
+        message.update(body)
         return message
 
     def format_json_message_iter(self):
@@ -273,6 +220,84 @@ class QueryResponse(TrellisMessage):
         return node_dict
 
 
+class MessageReader(object):
+
+    def __init__(
+                 self, 
+                 context, 
+                 event):
+
+        self.context = context
+
+        pubsub_message = base64.b64decode(event['data']).decode('utf-8')
+        data = json.loads(pubsub_message)
+
+        # Not sure what cases caused me to add this to db-query,
+        # but keeping it in because I don't see any harm
+        if type(data) == str:
+            logging.warn("Message data not correctly loaded as JSON. " +
+                         "Used eval to convert from string.")
+            data = eval(data)
+
+        seed_id = int(data['header'].get('seedId'))
+        previous_event_id = int(data['header'].get('previousEventId'))
+        if not previous_event_id:
+            previous_event_id = seed_id
+
+        self.context = context
+        self.message_kind = data['header']['messageKind']
+        self.header = data['header']
+        self.body = data['body']
+        self.sender = data['header']['sender']
+        self.event_id = context.event_id
+        self.seed_id = seed_id
+        self.previous_event_id = previous_event_id
+
+
+class QueryRequestReader(MessageReader):
+
+    def __init__(
+                 self,
+                 context,
+                 event):
+
+        super().__init__(context, event)
+
+        if self.message_kind != 'queryRequest':
+            return ValueError
+
+        self.query_name = self.body['queryName']
+        self.query_parameters = self.body['queryParameters']
+        self.custom = bool(self.body['custom'])
+
+        # Should only be populated for custom queries
+        if self.custom:
+            self.query = self.body['query']
+            self.write_transaction = bool(self.body['writeTransaction'])
+            self.publish_to = self.body.get('publishTo')
+            self.returns = self.body.get('returns')
+
+
+class QueryResponseReader(MessageReader):
+
+    def __init__(
+                 self,
+                 context,
+                 event):
+
+        super().__init__(context, event)
+        
+        if self.message_kind != 'queryResponse':
+            return ValueError
+
+        self.query_name = self.body['queryName']
+        self.result_summary = self.body['resultSummary']
+        self.nodes = self.body['nodes']
+        self.relationships = self.body['relationships']
+
+
+
+"""
 class JobLauncherResponse(TrellisMessage):
 
     def __init__(self,
@@ -312,3 +337,4 @@ class JobLauncherResponse(TrellisMessage):
            }
         }
         return message
+"""
\ No newline at end of file
